//---------------------------------------------------------------------------
// TestNonSpatialDispersal
//---------------------------------------------------------------------------
#if !defined(TestNonSpatialDispersal_H)
  #define TestNonSpatialDispersal_H

  #include <cppunit/TestFixture.h>
  #include <cppunit/TestCaller.h>

  class clGridBase;
  class clSimManager;

  /**
   This class tests the non-spatial dispersal behavior.  It writes out
   parameter files and uses them as input to perform setup; it then sets up
   the desired initial conditions, causes the behavior to run, then tests its
   output.  There will be two normal processing runs.  Both use deterministic
   seed numbers, which should equal the expected mean +- 1.
   */
  class clTestNonSpatialDispersal : public CppUnit::TestFixture
  {

  public:

  /**
  * Rolls together all the non-spatial dispersal tests.
  */
  void TestNonSpatialDispersal();

  /**
    Creates a test suite of all the test cases in this object.  I'm putting
    it here so it will be easier to see and add new cases to.

    @return The test suite.
  */

/*    static CppUnit::Test * suite()
    {
      CppUnit::TestSuite * suite = new CppUnit::TestSuite("TestNonSpatialDispersal");

      suite->addTest(
           new CppUnit::TestCaller < clTestNonSpatialDispersal > ("TestNonSpatialDispersal1", & clTestNonSpatialDispersal::TestNonSpatialDispersal1));
      suite->addTest(
           new CppUnit::TestCaller < clTestNonSpatialDispersal > ("TestNonSpatialDispersal2", & clTestNonSpatialDispersal::TestNonSpatialDispersal2));

      return suite;
    }; */

  private:

  /**
   Tests non-spatial dispersal, run 1.  It will cause a parameter file to
   be written and then passed to a clSimManager object.  That
   object will read it.  Then the non-spatial dispersal behavior is run.
   The number of seeds per grid cell is compared with expected values.
  */
    void TestNonSpatialDispersal1(clSimManager *p_oSimManager);

   /**
    Tests non-spatial dispersal, run 2.  It will cause a parameter file to
    be written and then passed to a clSimManager object.  That
    object will read it.  Then the non-spatial dispersal behavior is run.
    The number of seeds per grid cell is compared with expected values.
   */
    void TestNonSpatialDispersal2(clSimManager *p_oSimManager);

    /** Verifies the seedling counts for TestNonSpatialDispersal1.
    * @param p_oSeedGrid Dispersed seeds grid.
    */
    void VerifyXMLFile1(clGridBase * p_oSeedGrid);

    /** Verifies the seedling counts for TestNonSpatialDispersal2.
    * @param p_oSeedGrid Dispersed seeds grid.
    */
    void VerifyXMLFile2(clGridBase * p_oSeedGrid);

    /**
     Writes a parameter file to direct testing using the first set of testing
     conditions.  This contains all tree information and disperse information.
     It also has a tree map so we'll be dealing with a known list of trees.
     The timestep length is 1 year.

     @return Filename written.
    */
    char * WriteXMLFile1();

    /**
    Writes a parameter file to direct testing using the second set of testing
    conditions (same as the first set, just different number of years per
    timestep).  This contains all tree information and disperse information.
    It also has a tree map so we'll be dealing with a known list of trees.  The
    timestep length is three years.

    @return Filename written.
    */
    char * WriteXMLFile2();
};
//---------------------------------------------------------------------------
#endif // TestNonSpatialDispersal_H
