//---------------------------------------------------------------------------
// TestWindstorm.cpp
//---------------------------------------------------------------------------
#include "TestWindstorm.h"
#include "SimManager.h"
#include "GridBase.h"
#include "TreePopulation.h"
#include "Storm.h"
#include <fstream>
#include <sstream>
#include <math.h>


/////////////////////////////////////////////////////////////////////////////
//  TestWindstorm()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestWindstorm()
{
  clSimManager * p_oSimManager = new clSimManager( 3, 2, "" );

  TestNormalProcessingRun1( p_oSimManager );
  TestNormalProcessingRun2( p_oSimManager );
  TestNormalProcessingRun3( p_oSimManager );
  TestNormalProcessingRun4( p_oSimManager );
  TestNormalProcessingRun5( p_oSimManager );
  TestNormalProcessingRun6( p_oSimManager );
  TestNormalProcessingRun7( p_oSimManager );
  TestErrorProcessing( p_oSimManager );

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestErrorProcessing( clSimManager * p_oSimManager )
{

  try
  {
    p_oSimManager->ReadFile( WriteXMLErrorFile1() );
    CppUnit::Asserter::fail( "TestWindstorm error processing didn't catch error for WriteXMLErrorFile1." );
  }
  catch ( modelErr err )
  {
    if ( strstr( err.cFunction, "clWindstorm::ReadParFile" ) == NULL )
    {
      CppUnit::Asserter::fail( "TestWindstorm error processing caught wrong error for WriteXMLErrorFile1." );
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile( WriteXMLErrorFile2() );
    CppUnit::Asserter::fail( "TestWindstorm error processing didn't catch error for WriteXMLErrorFile2." );
  }
  catch ( modelErr err )
  {
    if ( strstr( err.cFunction, "clWindstorm::GetDeadCodes" ) == NULL )
    {
      CppUnit::Asserter::fail( "TestWindstorm error processing caught wrong error for WriteXMLErrorFile2." );
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile( WriteXMLErrorFile3() );
    CppUnit::Asserter::fail( "TestWindstorm error processing didn't catch error for WriteXMLErrorFile3." );
  }
  catch ( modelErr err )
  {
    if ( strstr( err.cFunction, "clWindstorm::ReadParFile" ) == NULL )
    {
      CppUnit::Asserter::fail( "TestWindstorm error processing caught wrong error for WriteXMLErrorFile3." );
    }
    ;
  }
  
  try
  {
    p_oSimManager->ReadFile( WriteXMLErrorFile4() );
    CppUnit::Asserter::fail( "TestWindstorm error processing didn't catch error for WriteXMLErrorFile4." );
  }
  catch ( modelErr err )
  {
    if ( strstr( err.cFunction, "clWindstorm::ReadParFile" ) == NULL )
    {
      CppUnit::Asserter::fail( "TestWindstorm error processing caught wrong error for WriteXMLErrorFile4." );
    }
    ;
  }
  
  try
  {
    p_oSimManager->ReadFile( WriteXMLErrorFile5() );
    CppUnit::Asserter::fail( "TestWindstorm error processing didn't catch error for WriteXMLErrorFile5." );
  }
  catch ( modelErr err )
  {
    if ( strstr( err.cFunction, "clWindstorm::GetDeadCodes" ) == NULL )
    {
      CppUnit::Asserter::fail( "TestWindstorm error processing caught wrong error for WriteXMLErrorFile5." );
    }
    ;
  }
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun1()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun1( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  clPackage *p_oStormPackage;
  stringstream sMessage;
  float fX, fY, fDiam, fVal, fExpected;
  int iSp1, iSp2, iSnagSp1, iSnagSp2, iSp, i;  
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile1() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  //1000 trees of DBH 4, Species 1, and DBH 5, Species 2.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::sapling, 4.0 );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::sapling, 5.0 );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  //1000 trees of DBH 4, Species 1, and DBH 5, Species 2.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::sapling, 4.0 );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::sapling, 5.0 );
  }

  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0; iSnagSp1 = 0; iSnagSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX < 8 && fY < 8 )
    {
      if (clTreePopulation::snag == p_oTree->GetType() && 0 == iSp) {
        iSnagSp1++;
      }
      else if (clTreePopulation::snag == p_oTree->GetType() && 1 == iSp) {
        iSnagSp2++;
      }
      else if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun1 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  fExpected = 1784.15;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 1828.83;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  fExpected = 215.841;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp1 ) / fExpected < 0.11 );
  fExpected = 171.164;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp2 ) / fExpected < 0.1 );
  



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0; iSnagSp1 = 0; iSnagSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if (clTreePopulation::snag == p_oTree->GetType() && 0 == iSp) {
        iSnagSp1++;
      }
      else if (clTreePopulation::snag == p_oTree->GetType() && 1 == iSp) {
        iSnagSp2++;
      }
      else if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun1 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  fExpected = 1846.58;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 1858.64;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  fExpected = 153.41;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp1 ) / fExpected < 0.1 );
  fExpected = 141.35;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp2 ) / fExpected < 0.1 );
  
  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  //Verify that there's one storm package for the grid
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
  
  //Verify the storm results
  fExpected = 0.2;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 10.685;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 8.539;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  fExpected = 246.168;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 208.348;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun2()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun2( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  clPackage *p_oStormPackage;
  stringstream sMessage;
  float fX, fY, fDiam, fVal, fExpected;
  int iSp1, iSp2, iSnagSp1, iSnagSp2, iSp, i;  
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile2() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  //1000 trees of DBH 4, Species 1, and DBH 5, Species 2.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::sapling, 4.0 );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::sapling, 5.0 );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  //1000 trees of DBH 4, Species 1, and DBH 5, Species 2.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::sapling, 4.0 );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::sapling, 5.0 );
  }

  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0; iSnagSp1 = 0; iSnagSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX < 8 && fY < 8 )
    {
      if (clTreePopulation::snag == p_oTree->GetType() && 0 == iSp) {
        iSnagSp1++;
      }
      else if (clTreePopulation::snag == p_oTree->GetType() && 1 == iSp) {
        iSnagSp2++;
      }
      else if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun2 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  fExpected = 1164.829;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 1605.774;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  fExpected = 835.170;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp1 ) / fExpected < 0.1 );
  fExpected = 394.225;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp2 ) / fExpected < 0.1 );
  



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0; iSnagSp1 = 0; iSnagSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if (clTreePopulation::snag == p_oTree->GetType() && 0 == iSp) {
        iSnagSp1++;
      }
      else if (clTreePopulation::snag == p_oTree->GetType() && 1 == iSp) {
        iSnagSp2++;
      }
      else if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun1 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  fExpected = 1512.341;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 1791.865;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  fExpected = 487.658;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp1 ) / fExpected < 0.1 );
  fExpected = 208.134;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp2 ) / fExpected < 0.1 );
  
  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  //Verify that there's one storm package for the grid
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
  
  //Verify the storm results
  fExpected = 0.8;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 40.990;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 19.274;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  fExpected = 881.885;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 401.573;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun3()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun3( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  clPackage *p_oStormPackage;
  stringstream sMessage;
  float fX, fY, fDiam, fVal, fExpected;
  int iSp1, iSp2, iSnagSp1, iSnagSp2, iSp, i;  
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile3() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0; iSnagSp1 = 0; iSnagSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX < 8 && fY < 8 )
    {
      if (clTreePopulation::snag == p_oTree->GetType() && 0 == iSp) {
        iSnagSp1++;
      }
      else if (clTreePopulation::snag == p_oTree->GetType() && 1 == iSp) {
        iSnagSp2++;
      }
      else if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun3 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  fExpected = 482.183;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 569.383;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  fExpected = 517.816;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp1 ) / fExpected < 0.1 );
  fExpected = 430.616;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp2 ) / fExpected < 0.1 );
  



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0; iSnagSp1 = 0; iSnagSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if (clTreePopulation::snag == p_oTree->GetType() && 0 == iSp) {
        iSnagSp1++;
      }
      else if (clTreePopulation::snag == p_oTree->GetType() && 1 == iSp) {
        iSnagSp2++;
      }
      else if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun3 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  fExpected = 606.761;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 633.046;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  fExpected = 393.238;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp1 ) / fExpected < 0.1 );
  fExpected = 366.953;
  CPPUNIT_ASSERT( fabs( fExpected - iSnagSp2 ) / fExpected < 0.1 );
  
  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  //Verify that the first storm's results
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  fExpected = 0.2;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 10.685;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 8.539;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  fExpected = 246.168;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 208.348;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  //Verify the second storm's results
  p_oStormPackage = p_oStormPackage->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  fExpected = 0.2;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 8.411;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 7.092;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  fExpected = 199.419;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 175.495;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  //Verify the third storm's results
  p_oStormPackage = p_oStormPackage->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  fExpected = 0.2;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 6.622;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 5.890;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.12);
  
  fExpected = 161.782;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 147.868;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  //Verify that that's the last one
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun4()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun4( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  clPackage *p_oStormPackage;
  stringstream sMessage;
  float fX, fY, fDiam, fVal, fExpected;
  int iSp1, iSp2, iSp, i;  
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile4() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::sapling, 4.0 );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::sapling, 5.0 );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::sapling, 4.0 );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::sapling, 5.0 );
  }

  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );
    
    CPPUNIT_ASSERT(clTreePopulation::snag != p_oTree->GetType());

    if ( fX < 8 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun4 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  fExpected = 1579.114;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 1767.597;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );  



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun4 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  fExpected = 1760.457;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 1838.634;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
  
  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  //Verify that there's one storm package for the grid
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
  
  //Verify the storm results
  fExpected = 0.4;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 20.636;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 11.492;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  fExpected = 440.285;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 262.512;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun5()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun5( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  stringstream sMessage;
  float fX, fY, fDiam;
  int iSp1, iSp2, iSp, i;  
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile5() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );
    
    CPPUNIT_ASSERT(clTreePopulation::snag != p_oTree->GetType());

    if ( fX < 8 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun5 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  CPPUNIT_ASSERT( 1000 == iSp1 );
  CPPUNIT_ASSERT( 1000 == iSp2 );  



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun5 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  CPPUNIT_ASSERT( 1000 == iSp1 );
  CPPUNIT_ASSERT( 1000 == iSp2 );  
  
  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  //Verify that there's no storm package for the grid
  CPPUNIT_ASSERT(NULL == p_oResultsGrid->GetFirstPackageOfCell(0, 0));
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun6()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun6( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  clPackage *p_oStormPackage;
  stringstream sMessage;
  float fX, fY, fDiam, fVal, fExpected;
  int iSp1, iSp2, iSp, i;  
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile6() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );
    
    CPPUNIT_ASSERT(clTreePopulation::snag != p_oTree->GetType());

    if ( fX < 8 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun6 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  CPPUNIT_ASSERT( 1000 == iSp1 );
  CPPUNIT_ASSERT( 1000 == iSp2 ); 



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun6 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  CPPUNIT_ASSERT( 1000 == iSp1 );
  CPPUNIT_ASSERT( 1000 == iSp2 ); 
  
  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  //Verify that there's no storm package for the grid
  CPPUNIT_ASSERT(NULL == p_oResultsGrid->GetFirstPackageOfCell(0, 0));
  
  //>>>>>>>>>>>>Run another timestep<<<<<<<<<<<<<<<<<<<<<<<
  p_oSimManager->RunSim( 1 );

  //Get all trees
  p_oAllTrees = p_oPop->Find( "all" );

  //>>>>>>>>>>>>Separate out and only do cell 0,0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );
    
    CPPUNIT_ASSERT(clTreePopulation::snag != p_oTree->GetType());

    if ( fX < 8 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun6 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Validate the counts
  fExpected = 458.513;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 731.744;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );  



  //>>>>>>>>>>>>Separate out and only do cell 1, 0<<<<<<<<<<<<<<<<<<<<<<<
  iSp1 = 0; iSp2 = 0;
  p_oAllTrees->StartOver();
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iSp = p_oTree->GetSpecies();

    //Get X and Y
    p_oTree->GetValue( p_oPop->GetXCode( iSp, p_oTree->GetType() ), & fX );
    p_oTree->GetValue( p_oPop->GetYCode( iSp, p_oTree->GetType() ), & fY );

    if ( fX > 8 && fX < 16 && fY < 8 )
    {
      if ( 0 == iSp )
      {
        iSp1++;
      }
      else if ( 1 == iSp )
      {
        iSp2++;
      }
      else
      {
        //Error
        sMessage << "TestNormalProcessingRun6 is panicking.  It found a "
                 << "tree it can't figure out.";
        CppUnit::Asserter::fail( sMessage.str() );
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  fExpected = 706.56;
  CPPUNIT_ASSERT( fabs( fExpected - iSp1 ) / fExpected < 0.1 );
  fExpected = 827.801;
  CPPUNIT_ASSERT( fabs( fExpected - iSp2 ) / fExpected < 0.1 );
    
  //Verify that there's one storm package for the grid
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT(NULL != p_oStormPackage);
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
  
  //Verify the storm results
  fExpected = 0.5;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
  
  fExpected = 26.500;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 13.218;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  
  fExpected = 556.617;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
  fExpected = 293.635;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal)/fExpected < 0.1);
}

/////////////////////////////////////////////////////////////////////////////
// TestNormalProcessingRun7()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::TestNormalProcessingRun7( clSimManager * p_oSimManager )
{
  using namespace std;
  clTreePopulation * p_oPop;
  clGridBase * p_oResultsGrid;
  clPackage *p_oStormPackage;
  stringstream sMessage;
  float fX, fY, fDiam, fVal, fExpected,
        fTS1BA0 = 0, fTS1BA1 = 0, fTS1Dens0 = 0, fTS1Dens1 = 0;
  int i;
  
  //Feed our file to the sim manager
  p_oSimManager->ReadFile( WriteXMLFile7() );

  //Get the tree population
  p_oPop = ( clTreePopulation * ) p_oSimManager->GetPopulationObject( "treepopulation" );

  //Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.
  fX = 1;  fY = 1;
  fDiam = 30;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  //Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.
  fX = 9; fY = 1;
  fDiam = 8;
  for ( i = 0; i < 1000; i++ )
  {
    p_oPop->CreateTree( fX, fY, 0, clTreePopulation::adult, fDiam );
    p_oPop->CreateTree( fX, fY, 1, clTreePopulation::adult, fDiam );
  }

  p_oSimManager->RunSim( 1 );

  //Get the "Windstorm Results" grid object
  p_oResultsGrid = p_oSimManager->GetGridObject( "Windstorm Results" );
  CPPUNIT_ASSERT( p_oResultsGrid != NULL );
  
  //Verify that there is only one grid per cell
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberXCells());
  CPPUNIT_ASSERT(1 == p_oResultsGrid->GetNumberYCells());
  
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT (NULL != p_oStormPackage);
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
  
  //Verify the severity
  fExpected = 0.1;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
    
  //Collect the other values - we don't care what they are, we'll just save
  //them for comparison with timestep 2
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fTS1BA0);
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fTS1BA1);
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fTS1Dens0);
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fTS1Dens1);
  
  //Run another timestep and verify that it has results that are different
  p_oSimManager->RunSim( 1 );
  
  p_oStormPackage = p_oResultsGrid->GetFirstPackageOfCell(0, 0);
  CPPUNIT_ASSERT (NULL != p_oStormPackage);
  CPPUNIT_ASSERT(NULL == p_oStormPackage->GetNextPackage());
  
  //Verify the severity
  fExpected = 0.1;
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("severity"), &fVal);
  CPPUNIT_ASSERT(fabs(fExpected - fVal) < 0.001);
    
  //Collect the other values and make sure they're different from timestep 1
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fTS1BA0 - fVal) > 0.01);
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("ba_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fTS1BA1 - fVal) > 0.01);
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fTS1Dens0 - fVal) > 5);
  p_oStormPackage->GetValue(p_oResultsGrid->GetPackageFloatDataCode("density_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fTS1Dens1 - fVal) > 5);
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.8</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>1</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.4</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile6()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>3</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.5</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>2</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile7()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile7()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>3</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.1</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile8()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLFile8()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.07</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;

}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLErrorFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">-5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLErrorFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  
  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Windstorm</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>";

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLErrorFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>-5</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLErrorFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  WriteNoSnagBehaviorList(oOut);

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>1.5</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestWindstorm::WriteXMLErrorFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  std::fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonStuff(oOut);
  
  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Windstorm</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>";

  oOut << "<windstorm>"
       << "<ws_severityReturnInterval1>0.2</ws_severityReturnInterval1>"
       << "<ws_severityReturnInterval5>0</ws_severityReturnInterval5>"
       << "<ws_severityReturnInterval10>0</ws_severityReturnInterval10>"
       << "<ws_severityReturnInterval20>0</ws_severityReturnInterval20>"
       << "<ws_severityReturnInterval40>0</ws_severityReturnInterval40>"
       << "<ws_severityReturnInterval80>0</ws_severityReturnInterval80>"
       << "<ws_severityReturnInterval160>0</ws_severityReturnInterval160>"
       << "<ws_severityReturnInterval320>0</ws_severityReturnInterval320>"
       << "<ws_severityReturnInterval640>0</ws_severityReturnInterval640>"
       << "<ws_severityReturnInterval1280>0</ws_severityReturnInterval1280>"
       << "<ws_severityReturnInterval2560>0</ws_severityReturnInterval2560>"
       << "<ws_minDBH>"
       << "<ws_mdVal species=\"Species_1\">5</ws_mdVal>"
       << "<ws_mdVal species=\"Species_2\">6</ws_mdVal>"
       << "</ws_minDBH>"
       << "<ws_stmInterceptA>"
       << "<ws_siaVal species=\"Species_1\">-2.261</ws_siaVal>"
       << "<ws_siaVal species=\"Species_2\">-1.96</ws_siaVal>"
       << "</ws_stmInterceptA>"
       << "<ws_stmIntensCoeffC>"
       << "<ws_sicVal species=\"Species_1\">1.14</ws_sicVal>"
       << "<ws_sicVal species=\"Species_2\">0.19</ws_sicVal>"
       << "</ws_stmIntensCoeffC>"
       << "<ws_stmDBHExpB>"
       << "<ws_sdebVal species=\"Species_1\">0.426</ws_sdebVal>"
       << "<ws_sdebVal species=\"Species_2\">0.679</ws_sdebVal>"
       << "</ws_stmDBHExpB>"
       << "<ws_stmTSToStartStorms>0</ws_stmTSToStartStorms>"
       << "</windstorm>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::WriteCommonStuff(std::fstream &oOut)
{
  using namespace std;
  oOut << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>150</plot_lenX>"
       << "<plot_lenY>100</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">5</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">5</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>";
}


/////////////////////////////////////////////////////////////////////////////
// WriteSnagBehaviorList()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::WriteSnagBehaviorList(std::fstream &oOut)
{
  using namespace std;
  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Windstorm</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>";
}

/////////////////////////////////////////////////////////////////////////////
// WriteNoSnagBehaviorList()
/////////////////////////////////////////////////////////////////////////////
void clTestWindstorm::WriteNoSnagBehaviorList(std::fstream &oOut)
{
  using namespace std;
  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>Windstorm</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>";
}
